//==============================================================================
// PROJECT      = L_Line - L2 demo
// MODULE       = rtc.C
// SHORT DESC.  = real time clock implementation
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================
// Modified by: Sven Killig
//              http://sven.killig.de/78K0
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "io78f0397.h"
#include "lcd.h"
#include "defines.h"
#include <intrinsics.h>

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
void Wait(unsigned char Number);

extern __callt void  BCD_calc(unsigned char data);

void buzzer();

//-----------------------------------------------------------------------------
// External constants
//-----------------------------------------------------------------------------
extern const char *s_clear;

extern const char *s_time;
extern const char *s_set;
extern const char *s_mode;
extern const char *s_alarm;

extern const char *timeAM;
extern const char *time24;

//-----------------------------------------------------------------------------
// External variables
//-----------------------------------------------------------------------------
extern __saddr volatile unsigned char sw3_in;

extern struct    {unsigned char s100;
                  unsigned char s10;
                  unsigned char s1;}bcd;

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
__saddr char clk_mode;
__saddr char clk_stop=1;

__saddr volatile unsigned char clk_sec05;

volatile struct {
          unsigned char sec;
          unsigned char min;
          unsigned char hour;
         }clock;

//-----------------------------------------------------------------------------
// Module:   rtc_display
// Description: Display current time
//-----------------------------------------------------------------------------
void rtc_display(unsigned char set)
{
     unsigned char hour=clock.hour;

     if(clk_mode)                                             // Converting from 24 hour clock to AM/PM
     {
        if(clock.hour<12) LCD_string("AM",6);                 // between 01:00 and 11:59, straight conversion to AM
        else LCD_string("PM",6);                              // between 12:00 and 12:59, straight conversion to PM
        if(clock.hour==0)  hour+=12;                          // between 00:00 and 00:59, add 12 hours
        if((clock.hour>=13)&&(clock.hour<=23)) hour-=12;      // between 13:00 and 23:59, subtract 12 hours
     }
     if(clk_sec05)                            // for flashing clock colons and clock values during adjustment
     {
        if(clk_mode) BCD_calc(hour);
        else BCD_calc(clock.hour);
        LCD_putc (0, bcd.s10);
        LCD_putc (1, bcd.s1);
        LCD_putc (2, 0xfd);
        BCD_calc(clock.min);
        LCD_putc (3, bcd.s10);
        LCD_putc (4, bcd.s1);
        BCD_calc(clock.sec);
        if((!clk_mode)/*&&(!set)*/)
        {
          LCD_putc (5, 0xfd);
          LCD_putc (6, bcd.s10);
          LCD_putc (7, bcd.s1);
        }
      }
      else                                     // flash clock colons/values each 0.5s
      {
        if((!clk_stop) && set==0)
        { LCD_putc (2, 0xf0); LCD_putc (5, 0xf0); }
        else if(set==1)
        { LCD_putc (0, 0xf0); LCD_putc (1, 0xf0); }
        else if(set==2)
        { LCD_putc (3, 0xf0); LCD_putc (4, 0xf0); }
        else if(set==3)
        { LCD_putc (6, 0xf0); LCD_putc (7, 0xf0); }
      }
     if((!clk_stop) && clock.hour==0 && clock.min==0 && clock.sec==0) {
       clk_stop=1;
       buzzer();
     }
}

//-----------------------------------------------------------------------------
// Module:   rtc_adjust
// Description: Adjust time
//-----------------------------------------------------------------------------
void rtc_adjust()
{
  unsigned char set=1;

  clock.sec=0;
  clk_stop=1;

  while(1)
    {
      rtc_display(set);
      switch(sw3_in)
      {
       case UP:    if(set==1)
                   {      clock.hour++;
                          if(clock.hour==100) clock.hour=0;
                   }
                   else if(set==2)
                   {      clock.min++;
                          if(clock.min==60) clock.min=0;
                   }
                   else if(set==3)
                   {      clock.sec++;
                          if(clock.sec==60) clock.sec=0;
                   }
                   break;
       case DOWN:  if(set==1)
                   {
                          clock.hour--;
                          if(clock.hour==0xff) clock.hour=99;
                   }
                   else if(set==2)
                   {
                          clock.min--;
                          if(clock.min==0xff) clock.min=59;
                   }
                   else if(set==3)
                   {
                          clock.sec--;
                          if(clock.sec==0xff) clock.sec=59;
                   }
                   break;
       case RIGHT: if(set<3) set++;
                   break;
       case LEFT:  if(set>1) set--;
                   break;
       case SELECT:
                   sw3_in=0;
                   clk_stop=0;
                   LCD_string(&s_clear[0],0);
                   return;

      }
      sw3_in=0;
   }
}

//-----------------------------------------------------------------------------
// Module:   rtc
// Description: Control rtc
//-----------------------------------------------------------------------------
void rtc()
{
  unsigned char set=0;

  while(1)
  {
      switch(sw3_in)
      {
       case UP:   set--;
                  LCD_string(&s_clear[0],0);
                  if(set==0xff) set=2;
                  break;

       case DOWN: set++;
                  LCD_string(&s_clear[0],0);
                  if(set==0x3) set=0;
                  break;

       case RIGHT:
                  sw3_in=0;
                  switch(set)
                  {
                   case 0x00: while(sw3_in!=LEFT)
  			      {
                                rtc_display(0);
                                __stop();
                              }
    			      sw3_in=0;
			      break;	

                   case 0x01: rtc_adjust();
                              break;
                   case 0x02:
                              while(sw3_in!=LEFT)
                              {
                                switch(sw3_in)
                                {
                                    case UP:
                                    case DOWN: clk_mode=!clk_mode;
                                               break;
                                }
                                sw3_in=0;
                                if(clk_mode) LCD_string(&timeAM[0],0);
                                else LCD_string(&time24[0],0);
                              }
                              break;
                   }
                   break;
       case LEFT: sw3_in=0;
                  return;

      }
      sw3_in=0;

      switch(set)
      {
      case 0x00: LCD_string(&s_time[0],0);
                 break;
      case 0x01: LCD_string(&s_set[0],0);
                 break;
      case 0x02: LCD_string(&s_mode[0],0);
                 break;
      }
  }
}


//-----------------------------------------------------------------------------
// ISR:      isr_INTWT_vect
// Function: Interrupt service routine of WatchTimer
//-----------------------------------------------------------------------------
#pragma vector = INTWT_vect
__interrupt void isr_INTWT(void)
{
  clk_sec05 = !clk_sec05 ;
   if((clk_sec05)&&(!clk_stop))
   {
     clock.sec--;				
     if(clock.sec==0xff)
     {
      clock.sec=59;
      clock.min--;
      if(clock.min==0xff)
      {
        clock.min=59;
        clock.hour--;
        if(clock.hour==0xff)
        {
          clock.hour=99;
        }
      }
     }
   }
}

